const chalk = require('chalk');
const metadata = require('../../metadata/metadata.js');
const { analyzeFuzzy, removeFromSet } = require('../search.js');

/**
 * Validate Attribute
 * @param {Object} obj The object from the first Promise that has valuable information about the custom prop
 * @todo Skip scope validation based on stylelint option
 * @returns {Object} Validation object
 */
const validateGlobalAttribute = (obj) => {
  if (obj.validate.scope.isComponent || obj.validate.scope.isShared) return { skip: true };
  const results = analyzeFuzzy(obj).filter((item) => item.word);

  if (obj.keys.attribute) {
    return { valid: true, received: obj.groups[obj.keys.attribute] };
  }
  // We have a previous customMessage, meaning we have already failed a validation
  else if (obj.customMessage.length > 0) {
    return { skip: true };
    // We have unknown keys, lets check if they are optional
  } else if (obj.fuzzyKeys.size > 0 && results.length > 0) {
    let valid;
    results.map((item) => {
      if (obj.keys.context && obj.keys.context + 2 === obj.keys.range) {
        obj.keys.attribute = obj.keys.context + 1;
        obj.fuzzyKeys = removeFromSet(obj.fuzzyKeys, obj.keys.attribute);
        valid = {
          valid: false,
          expected: metadata.global.valid.attribute,
          received: item.word,
        };
      }
      // nothing so far
      else {
        valid = { skip: true };
      }
    });
    return valid;
  } else {
    // This is an optional key so skip
    return { skip: true };
  }
};

module.exports = validateGlobalAttribute;
