const chalk = require('chalk');
const metadata = require('../../metadata/metadata.js');
const { analyzeFuzzy, removeFromSet } = require('../search');

/**
 * Validate Category
 * @param {Object} obj The object from the first Promise that has valuable information about the custom prop
 * @todo Skip scope validation based on stylelint option
 * @returns {Object} Validation object
 */
const validateGlobalCategory = (obj) => {
  if (obj.validate.scope.isComponent || obj.validate.scope.isShared) return { skip: true };
  const results = analyzeFuzzy(obj).filter((item) => item.word);

  if (obj.keys.category && obj.lastKey !== 2) {
    return { valid: true, received: obj.groups[obj.keys.category] };
  } else if (obj.keys.category && obj.lastKey === 2) {
    obj.customMessage.push(
      `We received "${chalk.bold(
        obj.groups[obj.keys.category],
      )}" as the last value in the custom property. A global styling hook requires a numerical range as the last value.`,
    );
    return {
      valid: false,
      expected: obj.groups[obj.keys.category],
    };
  } else if (obj.fuzzyKeys.size > 0 && results.length > 0) {
    let valid;
    results.map((item) => {
      if (obj.keys.scope === 1) {
        obj.keys.category = obj.keys.scope + 1;
        obj.fuzzyKeys = removeFromSet(obj.fuzzyKeys, obj.keys.category);
        valid = {
          valid: false,
          expected: metadata.global.valid.category,
          received: obj.groups[obj.keys.category],
        };
      }
    });
    return valid;
  } else {
    // This is a required key so throw an error if undefined
    return { valid: false, expected: metadata.global.valid.category };
  }
};

module.exports = validateGlobalCategory;
