const chalk = require('chalk');
const metadata = require('../../metadata/metadata.js');
const { analyzeFuzzy, removeFromSet } = require('../search.js');

/**
 * Validate Context
 * @param {Object} obj The object from the first Promise that has valuable information about the custom prop
 * @todo Skip scope validation based on stylelint option
 * @returns {Object} Validation object
 */
const validateGlobalContext = (obj) => {
  if (obj.validate.scope.isComponent || obj.validate.scope.isShared) return { skip: true };
  const results = analyzeFuzzy(obj).filter((item) => item.word);

  if (obj.keys.category === 2 && obj.keys.pairing && obj.keys.context && !obj.keys.role) {
    obj.customMessage.push(
      `We have determined you are trying to set a pairing to a context. Please set a role to the pairing. Available options are ${chalk.bold(
        metadata.global.valid.role,
      )}`,
    );
    return {
      valid: false,
      received: obj.groups[obj.keys.context],
    };
  }
  // We've passed glaring issues so we know its a valid context
  else if (obj.keys.context) {
    return { valid: true, received: obj.groups[obj.keys.context] };
  }
  // We have a previous customMessage, meaning we have already failed a validation
  else if (obj.customMessage.length > 0) {
    return { skip: true };
    // We have unknown keys, lets check if they are optional
  } else if (obj.fuzzyKeys.size > 0 && results.length > 0) {
    let valid;
    results.map((item) => {
      if (obj.keys.category === 2 && obj.keys.role && obj.groups[obj.keys.role] !== 'palette') {
        obj.keys.context = obj.keys.role + 1;
        obj.fuzzyKeys = removeFromSet(obj.fuzzyKeys, obj.keys.context);
        valid = {
          valid: false,
          expected: metadata.global.valid.context,
          received: item.word,
        };
      } else {
        valid = { skip: true };
      }
    });
    return valid;
  } else {
    // This is an optional key so skip
    return { skip: true };
  }
};

module.exports = validateGlobalContext;
