const chalk = require('chalk');
const metadata = require('../../metadata/metadata.js');
const { analyzeFuzzy, removeFromSet } = require('../search');

/**
 * Validate Property
 * @param {Object} obj The object from the first Promise that has valuable information about the custom prop
 * @todo Skip scope validation based on stylelint option
 * @returns {Object} Validation object
 */
const validateGlobalRole = (obj) => {
  if (obj.validate.scope.isComponent || obj.validate.scope.isShared) return { skip: true };
  const results = analyzeFuzzy(obj).filter((item) => item.word);

  if (obj.keys.role && obj.lastKey !== 3) {
    return { valid: true, received: obj.groups[obj.keys.role] };
  } else if (obj.keys.role && obj.lastKey === 3) {
    obj.customMessage.push(
      `We received "${chalk.bold(
        obj.groups[obj.keys.role],
      )}" as the last value in the custom property. A global styling hook requires a numerical range as the last value.`,
    );
    return {
      valid: false,
      expected: obj.groups[obj.keys.role],
    };
  }
  // We have a previous customMessage, meaning we have already failed a validation
  else if (obj.customMessage.length > 0) {
    return { skip: true };
    // We have unknown keys, lets check if they are optional
  } else if (obj.fuzzyKeys.size > 0 && results.length > 0) {
    let valid;
    results.map((item) => {
      if (obj.keys.category === 2 && obj.keys.pairing === 3 && obj.keys.range >= 5) {
        obj.keys.role = obj.keys.pairing + 1;
        obj.fuzzyKeys = removeFromSet(obj.fuzzyKeys, obj.keys.role);
        valid = {
          valid: false,
          expected: metadata.global.valid.role,
          received: item.word,
        };
      } else {
        valid = { skip: true };
      }
    });
    return valid;
  } else {
    // This is an optional key so skip
    return { skip: true };
  }
};

module.exports = validateGlobalRole;
