const metadata = require('../metadata/metadata.js');

/**
 * Check if the CSS custom property should be recognized by Stylelint
 *
 * @param {String} value - value to be evaluated
 * @param {String} privateSyntax - syntax that declares a custom property as private
 *
 * @returns {Boolean} - true if a valid CSS custom property and not a private variable
 */
const isValidCustomProperty = ({ value, privateSyntax }) => {
  const REQUIRED_PREFIX = '--';

  return typeof value === 'string' && value.startsWith(REQUIRED_PREFIX) && !value.includes(privateSyntax);
};

module.exports = isValidCustomProperty;
