const metadata = require('../metadata/metadata.js');

/**
 * Store keys based on metadata
 * @param {Object[]} arr An array of words from the css custom property
 * @returns {Object} An object of keys based on metadata
 */
const storeSearchableKeys = (arr) => {
  const local = {
    scope: null,
    context: null,
    variant: null,
    element: null,
    category: null,
    property: null,
    attribute: null,
    state: null,
    psuedoState: null,
  };

  arr.map((el, key) => {
    metadata.component.valid.scope.find((val) => {
      if (el === val) {
        local.scope = key;
      }
    });
    metadata.component.valid.category.find((val) => {
      if (el === val) {
        local.category = key;
      }
    });
    metadata.component.valid.property.find((val) => {
      if (el === val) {
        local.property = key;
      }
    });
    metadata.component.valid.attribute.find((val) => {
      if (el === val) {
        local.attribute = key;
      }
    });
    metadata.component.valid.state.find((val) => {
      if (el === val) {
        local.state = key;
      }
    });
    metadata.component.valid.psuedoState.find((val) => {
      if (el === val) {
        local.psuedoState = key;
      }
    });
  });
  return local;
};

/**
 * Store keys based on metadata
 * @param {Object[]} arr An array of words from the css custom property
 * @returns {Object} An object of keys based on metadata
 */
const storeSearchableGlobalKeys = (arr) => {
  const local = {
    scope: null,
    category: null,
    property: null,
    pairing: null,
    role: null,
    attribute: null,
    variant: null,
    range: null,
  };

  arr.map((el, key) => {
    metadata.global.valid.scope.find((val) => {
      if (el === val) {
        local.scope = key;
      }
    });
    metadata.global.valid.category.find((val) => {
      if (el === val) {
        local.category = key;
      }
    });
    metadata.global.valid.property.find((val) => {
      if (el === val) {
        local.property = key;
      }
    });
    metadata.global.valid.pairing.find((val) => {
      if (el === val) {
        local.pairing = key;
      }
    });
    metadata.global.valid.role.find((val) => {
      if (el === val) {
        local.role = key;
      }
    });
    metadata.global.valid.context.find((val) => {
      if (el === val) {
        local.context = key;
      }
    });
    metadata.global.valid.attribute.find((val) => {
      if (el === val) {
        local.attribute = key;
      }
    });
    metadata.global.valid.range.find((val) => {
      if (el === val) {
        local.range = key;
      }
    });
  });
  return local;
};

const analyzeFuzzy = (obj) => {
  const suggested = [];
  const checkFuzzyKeys = obj.fuzzyKeys.values();

  obj.fuzzyKeys.forEach((it) => {
    const suggest = {
      word: null,
      stored: null,
      key: null,
      lastKey: null,
    };
    if (obj.groups[checkFuzzyKeys.next().value]) {
      if (!Object.values(obj.keys).includes(it)) {
        suggest.word = obj.groups[it];
        suggest.stored = Object.values(obj.keys).includes(it);
        suggest.key = it;
        suggest.lastKey = it === obj.lastKey;
      }
    }
    suggested.push(suggest);
  });
  return suggested;
};

/**
 * Retrieve missing keys from initial index storage based on metadata
 * @param {*} set
 * @param {*} total
 * @returns {Object[]} Set of unused keys
 */
const getMissingKeys = (set, total) => {
  let keySet = [];
  Object.keys(set).map((key) => {
    if (set[key]) {
      keySet.push(set[key]);
    }
  });
  if (!keySet.includes(total)) {
    keySet = [...keySet, total];
  }
  const range = Array.from(Array(Math.max(...keySet)).keys());
  const missing = range
    .map((i) => {
      if (keySet.indexOf(i) < 0 && i > Math.min(...keySet)) {
        return i;
      }
    })
    .filter((f) => f);
  return missing;
};

const removeFromArray = (arr, value) => {
  return arr.filter((el) => el !== value);
};

const removeFromSet = (set, value) => {
  set.delete(value);
  return set;
};

module.exports = {
  analyzeFuzzy,
  getMissingKeys,
  removeFromArray,
  removeFromSet,
  storeSearchableKeys,
  storeSearchableGlobalKeys,
};
