const chalk = require('chalk');
const { analyzeFuzzy, removeFromSet } = require('./search');
const metadata = require('../metadata/metadata.js');

/**
 * Validate state
 * @param {Object} obj The object from the first Promise that has valuable information about the custom prop
 * @todo Skip scope validation based on stylelint option
 * @returns {Object} Validation object or definition to skip if not found
 */
const validateState = (obj) => {
  if (obj.validate.scope.isGlobal || obj.validate.scope.isShared) return { skip: true };

  const results = analyzeFuzzy(obj).filter((item) => item.word);

  if (obj.keys.state) {
    return { valid: true, received: obj.groups[obj.keys.state] };
  } else if (obj.fuzzyKeys.size > 0 && results.length > 0) {
    let valid;
    results.map((item) => {
      if (!item.lastKey && item.key === obj.keys.attribute + 1) {
        obj.keys.state = obj.keys.attribute + 1;
        obj.fuzzyKeys = removeFromSet(obj.fuzzyKeys, obj.keys.state);
        valid = {
          valid: false,
          expected: metadata.component.valid.state,
          received: obj.groups[obj.keys.state],
        };
      } else if (!obj.keys.state && item.lastKey && results.length === 2) {
        valid = { valid: false, received: obj.groups[item.key] };
        obj.customMessage.push(
          `We received "${chalk.bold(
            obj.groups[item.key],
          )}" and have medium confidence that this could be a state or a psuedoState. Possible values are ${chalk.bold(
            metadata.component.valid.state,
          )},${chalk.bold(metadata.component.valid.psuedoState)}.`,
        );
      }
      // fuzzy result already captured previously
      else if (
        (item.lastKey && obj.customMessage.length === 1) ||
        (!obj.validate.attribute.valid && obj.customMessage.length === 1)
      ) {
        valid = { skip: true };
      }
    });
    return valid;
  } else {
    return { skip: true };
  }
};

module.exports = validateState;
